/*
 ***********************************************************************************************
 *                                                                                              
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 
 *                                                                                              
 *  OSA is distributed under BSD license (see license.txt)                                      
 *                                                                                              
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro                                  
 *              http://picosa.narod.ru                                                          
 *                                                                                              
 *----------------------------------------------------------------------------------------------
 *                                                                                              
 *  File:       osa_tasks.h                                                                     
 *                                                                                              
 *  Programmer: Timofeev Victor                                                                 
 *              osa@pic24.ru, testerplus@mail.ru                                                
 *                                                                                              
 *  Definition: Services for work with tasks                                                    
 *                                                                                              
 *  History:    26.02.2009   Bug fixed: OS_Task_Replace did not reserve created task by fncall  
 *                           OS_ReserveTask renamed to OS_Task_Reserve                          
 *              01.04.2009   All processor specific services for tasks (create, replace and     
 *                           reserve) were moved into proc specific h-file in "port" folder     
 *              01.07.2009 - Removed brasses around parametr pTask in all macros (because CCS   
 *                           compiler exited with error on expression (pTask)->...              
 *              03.11.2009 - Service OS_Task_Replace_P added                                    
 *                                                                                              
 ***********************************************************************************************
 */



/************************************************************************************************
 *                                                                                              *
 *                                   T A S K S                                                  *
 *                                                                                              *
 ************************************************************************************************/

#define this_task   _OS_CurTask

/************************************************************************/
/*                                                                      */
/* Stop current task                                                    */
/*                                                                      */
/************************************************************************/

#define OS_Task_Stop()   OS_Task_Delete(this_task)


/************************************************************************/
/*                                                                      */
/* Delete task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Delete(pTask)                                           \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        pTask->State.bEnable = 0;                                       \
        if ((&(pTask)) == (&_OS_CurTask))                               \
        {                                                               \
            _OS_Return();                                               \
        }                                                               \
        _OS_CLR_IRP();          /*90922*/                               \
    }


/************************************************************************/
/*                                                                      */
/* Pause task                                                           */
/*                                                                      */
/************************************************************************/

#define OS_Task_Pause(pTask)                                            \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        pTask->State.bPaused = 1;                                       \
        if ((&(pTask)) == (&_OS_CurTask))                               \
        {                                                               \
            _OS_Return();                                               \
        }                                                               \
        _OS_CLR_IRP();          /*90922*/                               \
    }

/************************************************************************/
/*                                                                      */
/* Continue paused task                                                 */
/*                                                                      */
/************************************************************************/

#define OS_Task_Continue(pTask)                                         \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        pTask->State.bPaused = 0                                        \
        _OS_CLR_IRP();          /*90922*/                               \
    }


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

// See proc.spec. h-file in "port" folder.



/****************************************************************************/
/*                                                                          */
/* Create task                                                              */
/*                                                                          */
/****************************************************************************/

// See proc.spec. h-file in "port" folder.



/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

#define OS_Task_CreateP(priority, TaskAddr)      _OS_Task_Create(priority, TaskAddr)
#define OS_Task_Create_P(priority, TaskAddr)     _OS_Task_Create(priority, TaskAddr)

// If task created indirrectly then it is nessesary to insert in
// the main() service OS_ReserveTasks:
// see proc.spec. h-file in "port" folder.



/************************************************************************/
/*                                                                      */
/* Get/set current task priority                                        */
/*                                                                      */
/************************************************************************/

#ifdef __OSAMIKROC16__
#define OS_Task_GetPriority(pTask)      _OS_Task_GetPriority(pTask)
#else
#define OS_Task_GetPriority(pTask)      pTask->State.Priority
#endif

#define OS_Task_SetPriority(pTask, new_priority)                        \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        pTask->State.Priority = new_priority;                           \
        _OS_CLR_IRP();          /*90922*/                               \
    }



/************************************************************************/
/*                                                                      */
/* Get current task's descriptor address                                */
/*                                                                      */
/************************************************************************/

#define OS_GetCurTask()                 this_task


/************************************************************************/
/*                                                                      */
/* Check for task is paused                                             */
/*                                                                      */
/************************************************************************/

#ifdef __OSAMIKROC16__
#define OS_Task_IsPaused(pTask)         _OS_Task_IsPaused(pTask)
#else
#define OS_Task_IsPaused(pTask)         pTask->State.bPaused
#endif















